from dbtunnel.tunnels import DbTunnel
from dbtunnel.utils import process_file, execute


class StreamlitTunnel(DbTunnel):

    def __init__(self, script_path: str, port: int):
        super().__init__(port, "streamlit")
        self._script_path = script_path

    def _imports(self):
        try:
            import streamlit
            import nest_asyncio
        except ImportError as e:
            print("ImportError: Make sure you have nest_asyncio, streamlit installed. \n"
                  "pip install nest_asyncio streamlit")
            raise e

    def _display_url(self):
        # must end with a "/" for it to not redirect
        return f'<a href="{self._proxy_settings.proxy_url}">Click to go to {self._flavor} App!</a>'

    def _run(self):
        self.display()
        print("Starting server...", flush=True)
        import nest_asyncio
        nest_asyncio.apply()
        print(f"Use this link: \n{self._proxy_settings.proxy_url}")
        with process_file(self._script_path) as file_path:
            self.run_streamlit(file_path, self._port)

    @staticmethod
    def run_streamlit(path, port):
        import os
        import subprocess
        my_env = os.environ.copy()
        my_env["STREAMLIT_SERVER_PORT"] = f"{port}"
        my_env["STREAMLIT_SERVER_ADDRESS"] = "0.0.0.0"
        my_env["STREAMLIT_SERVER_HEADLESS"] = "true"
        subprocess.run(f"kill -9 $(lsof -t -i:{port})", capture_output=True, shell=True)

        print(f"Deploying streamlit app at path: {path} on port: {port}")
        cmd = [
            "streamlit",
            "run",
            path,
            "--browser.gatherUsageStats",
            "false",
            "--server.fileWatcherType",
            "none"
        ]
        print(f"Running command: {' '.join(cmd)}")
        for path in execute(cmd, my_env):
            print(path, end="")


def streamlit_patch_websockets_v2():
    from pathlib import Path
    import subprocess
    import streamlit
    p = Path(streamlit.__file__)
    _dir = (p.parent)
    process = subprocess.run(f"find {_dir} -type f -exec sed -i -e 's/\"stream\"/\"ws\"/g' {{}} \;",
                             capture_output=True, shell=True)
    return process.stdout.decode()



# Databricks notebook source
# MAGIC %pip install dbtunnel[streamlit] pyngrok

# COMMAND ----------

dbutils.library.restartPython()

# COMMAND ----------

import os

current_directory = os.getcwd()
script_path = current_directory + "/streamlit_example.py"

# COMMAND ----------

from dbtunnel import dbtunnel

dbtunnel.streamlit(script_path).run()

# COMMAND ----------





from dbtunnel.bokeh import BokehTunnel
from dbtunnel.chainlit import ChainlitAppTunnel
from dbtunnel.code_server import CodeServerTunnel
from dbtunnel.dash import DashAppTunnel
from dbtunnel.fastapi import FastApiAppTunnel
from dbtunnel.flask import FlaskAppTunnel
from dbtunnel.gradio import GradioAppTunnel
from dbtunnel.nicegui import NiceGuiAppTunnel
from dbtunnel.solara import SolaraAppTunnel
from dbtunnel.stable_diffusion_ui import StableDiffusionUITunnel
from dbtunnel.streamlit import StreamlitTunnel


class AppTunnels:

    @staticmethod
    def fastapi(app, port: int = 8080):
        return FastApiAppTunnel(app, port)

    @staticmethod
    def gradio(app, port: int = 8080):
        return GradioAppTunnel(app, port)

    @staticmethod
    def nicegui(app, storage_secret: str = "", port: int = 8080):
        return NiceGuiAppTunnel(app, storage_secret, port)

    @staticmethod
    def streamlit(path, port: int = 8080):
        return StreamlitTunnel(path, port)

    @staticmethod
    def stable_diffusion_ui(no_gpu: bool, port: int = 7860, enable_insecure_extensions: bool = False,
                            extra_flags: str = ""):
        # todo auto detect with torch
        return StableDiffusionUITunnel(no_gpu, port, enable_insecure_extensions, extra_flags)

    @staticmethod
    def bokeh(path, port: int = 8080):
        return BokehTunnel(path, port)

    @staticmethod
    def flask(app, port: int = 8080):
        return FlaskAppTunnel(app, port)

    @staticmethod
    def dash(app, port: int = 8080):
        return DashAppTunnel(app, port)

    @staticmethod
    def solara(path, port: int = 8080):
        return SolaraAppTunnel(path, port)

    @staticmethod
    def code_server(directory_path: str = None, repo_name: str = None, port: int = 9988):
        return CodeServerTunnel(directory_path, repo_name, port)

    @staticmethod
    def chainlit(script_path: str, port: int = 8000):
        return ChainlitAppTunnel(script_path, port)


dbtunnel = AppTunnels()



import os
import shutil
import subprocess
import tempfile
from contextlib import contextmanager
from typing import List


@contextmanager
def process_file(input_path):
    temp_dir = tempfile.mkdtemp()
    try:
        # Create a temporary directory

        # Build the destination path in the temporary directory
        temp_file_path = os.path.join(temp_dir, os.path.basename(input_path))

        # Copy the file to the temporary directory
        shutil.copy(input_path, temp_file_path)

        # Yield the temporary file path to the caller
        yield temp_file_path
    finally:
        # Cleanup: Remove the temporary directory and its contents
        shutil.rmtree(temp_dir, ignore_errors=True)


def execute(cmd: List[str], env):
    import subprocess
    popen = subprocess.Popen(cmd, stdout=subprocess.PIPE, universal_newlines=True, env=env)
    for stdout_line in iter(popen.stdout.readline, ""):
        yield stdout_line
    popen.stdout.close()
    return_code = popen.wait()
    if return_code:
        raise subprocess.CalledProcessError(return_code, cmd)

def pkill(process_name):
    try:
        subprocess.run(["pkill", process_name])
        print(f"Processes with name '{process_name}' killed successfully.")
    except subprocess.CalledProcessError as e:
        print(f"Error killing processes: {e}")


from dbtunnel.tunnels import DbTunnel
from dbtunnel.utils import execute


class ChainlitAppTunnel(DbTunnel):

    def _imports(self):
        try:
            import chainlit
            import nest_asyncio
        except ImportError as e:
            print("ImportError: Make sure you have chainlit installed. \n"
                  "pip install chainlit nest_asyncio")
            raise e

    def _run(self):
        if self._share is False:
            raise Exception("Chainlit app must be shared to run. Please use .share_to_internet_via_ngrok(...)")

        print("Starting chainlit...", flush=True)

        import os
        import subprocess
        my_env = os.environ.copy()
        subprocess.run(f"kill -9 $(lsof -t -i:{self._port})", capture_output=True, shell=True)

        cmd = ["chainlit", "run", self._chainlit_script_path, "-h", "--host", "0.0.0.0", "--port", f"{self._port}"]
        print(f"Running command: {' '.join(cmd)}")
        for path in execute(cmd, my_env):
            print(path, end="")

    def __init__(self, chainlit_script_path: str, port: int = 8000):
        super().__init__(port, "chainlit")
        self._chainlit_script_path = chainlit_script_path

    def _display_url(self):
        return None


